<?php
/*
 * Plugin-part for prefab widgets in Wordpress
 */

class EveryBoard_WidgetPrefab {

    public function __construct() {

        add_action( 'admin_menu', array( &$this, 'register_widgetprefab_submenu_page' ) );

        global $pagenow;

        if ( is_admin() && $pagenow == "edit.php" && isset( $_GET['page'] ) && $_GET["page"] == "widget-prefab" ) {
            add_action( 'admin_enqueue_scripts', array( &$this, 'prefab_enque_scripts') );
        }
        // Call for register of sidebar
        $this->_register_sidebar();

        if ( ! function_exists( 'wp_list_widgets' ) && is_admin() ) {
            require_once( ABSPATH . '/wp-admin/includes/widgets.php' );
        }

        // Register custom settings filters for widgets
        add_filter('in_widget_form', array(&$this,'widget_settings_callback'), 10, 3);
        add_filter('widget_update_callback', array(&$this,'widget_update_callback'), 10, 3);
    }

    public function prefab_enque_scripts(){
        wp_enqueue_script("widgetprefab", EVERYBOARD_BASE . "assets/dist/js/boardwidgets.min.ugly.js", array( 'ever-admin' ));
        wp_localize_script( 'widgetprefab', 'translation_boardwidgets', array(
            'remove_widget_confirm' => __("Should the widget be removed?", "everyboard")
        ));
        wp_enqueue_style( 'widgetprefab-style', EVERYBOARD_BASE . 'assets/dist/css/boardsettings.min.css' );
    }

    // Add custom fields on every widget, called by filter
    public function widget_settings_callback($widget, $return, $instance){
    ?>
        <hr style="border: 1px solid #ccc" />

        <?php echo __('Custom name:', 'everyboard'); ?> <input type="text" placeholder='<?php echo __('Name for EveryBoard...', 'everyboard'); ?>' name="<?php echo $widget->get_field_name('board_widget_name'); ?>" id="<?php echo $widget->get_field_id('board_widget_name'); ?>" class="board_widget_name" value="<?php echo isset( $instance['board_widget_name'] ) ? $instance['board_widget_name'] : ''; ?>" />
        <input type="hidden" id="orgin_name" name="orgin_name" value="<?php echo $widget->name; ?>" />

        <br /><br />

        <?php __('Active on boards:', 'everyboard'); ?><br />

    <?php
        $boards = $this->get_boards_from_widget($widget->id);

        if(count($boards) < 1) {
            echo __('Not active on any board.', 'everyboard');
        }

        $bt = "";
        foreach($boards as $board) {
            #print_r($board);
            $link = get_edit_post_link( $board->ID );
            $bt .= "<a href='".$link."' target='_blank'>".$board->{"post_title"} . "</a>, ";
        }

        $bt = substr($bt, 0, strlen($bt)-2);
        echo $bt;

        $return = null;
    }

    public function get_boards_from_widget($widgetid) {

        $board_args = array(
            'post_type'  => 'everyboard',
            'post_status' => 'any'
        );

        $boards = get_posts( $board_args );
        $found_boards = array();

        foreach($boards as $board) {
            $custom_fields = get_post_custom($board->ID);
            $json = isset( $custom_fields["everyboard_json"] ) ? $custom_fields["everyboard_json"][0] : '';
            $json = json_decode($json);

            if(isset($json->rows)) {
                foreach($json->rows as $row) {
                    if(isset($row->cols)) {
                        foreach($row->cols as $col) {
                            if(isset($col->content)) {
                                foreach($col->content as $content) {
                                    if(isset($content->type)) {
                                        if ($content->type == "widget") {
                                            if ($widgetid == $content->widget_id) {
                                                if (!array_key_exists($board->ID, $found_boards)) {
                                                    $found_boards[$board->ID] = $board;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $found_boards;
    }

    // Save custom field settings, called by filter
    public function widget_update_callback($instance, $new_instance, $old_instance) {
        $instance['board_widget_name'] = $new_instance['board_widget_name'];
        return $instance;
    }

    /*
	 * Function to get all prefab widgets
	 * @return widgets, array
	 */
    public static function get_widgets() {
        global $wp_registered_widgets;
        $sidebar_widgets = wp_get_sidebars_widgets();

        $widgets = array();
        if( !isset( $sidebar_widgets["everyboard_widgets"] ) ) { return $widgets; }

        $active_widgets = $sidebar_widgets["everyboard_widgets"];
        sort( $active_widgets );

        foreach($active_widgets as $widget) {
            if( isset( $wp_registered_widgets[$widget] ) ) {
                $settings = $wp_registered_widgets[$widget]["callback"][0]->get_settings();
                $settings = $settings[$wp_registered_widgets[$widget]["params"][0]["number"]];

                $widgets[] = array(
                    "name"          => isset($settings["board_widget_name"]) && strlen($settings["board_widget_name"]) > 0 ? $settings["board_widget_name"] : $wp_registered_widgets[$widget]["name"],
                    "description"   => "",
                    "id"            => $widget,
                    "orgin_name"    => $wp_registered_widgets[$widget]["name"]
                );
            }
        }

        return $widgets;
    }

    /*
	 * Function to get all prefab widgets
	 * @return widget, array
	 */
    public static function get_widget_by_id($widget_id) {
        global $wp_registered_widgets;
        $sidebar_widgets = wp_get_sidebars_widgets();

        $widgets = array();
        if(!isset($sidebar_widgets["everyboard_widgets"])) { return $widgets; }

        $active_widgets = $sidebar_widgets["everyboard_widgets"];
        if(!isset($wp_registered_widgets[$widget_id])){ return $widgets; }

        $settings = $wp_registered_widgets[$widget_id]["callback"][0]->get_settings();
        $settings = $settings[$wp_registered_widgets[$widget_id]["params"][0]["number"]];

        $widgets = array(
            "name"          => isset($settings["board_widget_name"]) && strlen($settings["board_widget_name"]) > 0 ? $settings["board_widget_name"] : $wp_registered_widgets[$widget_id]["name"],
            "description"   => "",
            "id"            => $widget_id,
            "orgin_name"    => $wp_registered_widgets[$widget_id]["name"]
        );

        return $widgets;
    }

    public static function get_widget_instance($widget_id, $reinitialize_widgets = false) {
        $ret = null;

        global $wp_registered_widgets;

        // If we have switched blogs we need to reinitialize widgets to get the active blogs widgets.
        if( $reinitialize_widgets ) {
            $old_widgets = $wp_registered_widgets;
            $wp_registered_widgets = array();
            do_action( 'widgets_init' );
        }

        $sidebar_widgets = wp_get_sidebars_widgets();
        if( isset( $wp_registered_widgets[$widget_id] ) && in_array( $widget_id, $sidebar_widgets["everyboard_widgets"] ) ) {
            $classname = get_class($wp_registered_widgets[$widget_id]["callback"][0]);
            if(!$wp_registered_widgets[$widget_id]["callback"][0]) {
                return null;
            } else {
                $settings = $wp_registered_widgets[$widget_id]["callback"][0]->get_settings();
                $settings = $settings[$wp_registered_widgets[$widget_id]["params"][0]["number"]];

                $ret =  array("classname" => $classname, "settings" => $settings);
            }
        }

        if( $reinitialize_widgets ) {
            $wp_registered_widgets = $old_widgets;
        }

        return $ret;
    }

    // Register our widget sidebar for prefabed widgets
    private function _register_sidebar() {
        register_sidebar( array(
            'name'          => 'EveryBoard Widgets',
            'id'            => 'everyboard_widgets',
            'before_widget' => '',
            'after_widget'  => '',
            'before_title'  => '',
            'after_title'   => '',
        ) );
    }


    public function register_widgetprefab_submenu_page() {
        add_submenu_page( 'edit.php?post_type=everyboard', 'Board Widgets', 'Board Widgets', 'manage_options', 'widget-prefab', array(&$this, 'admin_page') );
    }

    public function admin_page() {

        global $wp_registered_widgets;

        // Replace name for widgets

        foreach($wp_registered_widgets as &$widget) {
            $widgetObj = $widget["callback"][0];
            $settings = $widgetObj->get_settings();

            if(isset($widget["id"]) && isset($settings[$widget["params"][0]["number"]]["board_widget_name"]) && strlen($settings[$widget["params"][0]["number"]]["board_widget_name"]) > 0) {
                #echo $widget["name"] . " - " . $settings[$widget["params"][0]["number"]]["board_widget_name"] ."<br />";

                $widget["tmpname"] = $widget["name"];
                $widget["name"] = $settings[$widget["params"][0]["number"]]["board_widget_name"]; //. " - " . $widget["name"] . "";
            }
        }

        // TODO: Clean up js/css to files
        ?>
        <script type="text/javascript">
            (function($){
                $(document).ready(function(){
                    var avail_widgets = $("#widget-list > div");

                    $("#filter_widgets").keyup(function(){

                       avail_widgets.css("display", "block");

                       if($("#filter_widgets").val().length > 0) {
                            avail_widgets.filter(function(index){
                                return ($('h4', this).html().toLowerCase().indexOf($("#filter_widgets").val().toLowerCase()) < 0);
                            }).css("display", "none");
                       }

                    });
                });
            })(jQuery);
        </script>

        <?php wp_nonce_field( 'save-sidebar-widgets', '_wpnonce_widgets', false ); ?>

        <div class="wrap widgetprefab">
            <div id="icon-themes" class="icon32"><br /></div>
            <h2>Board widgets</h2>

            <div class="widget-liquid-left">
                <div id="widgets-left">
                    <div class="widgets-holder-wrap">
                        <div id='top_banner_widget' class=''>
                            <?php
                                global $wp_registered_sidebars;
                                $sidebar = "everyboard_widgets";
                                $registered_sidebar = $wp_registered_sidebars["everyboard_widgets"];

                                $wrap_class = 'widgets-holder-wrap';
                                if ( !empty( $registered_sidebar['class'] ) )
                                    $wrap_class .= ' sidebar-' . $registered_sidebar['class'];
                             ?>

                            <div class="<?php echo esc_attr( $wrap_class ); ?> active-widgets">
                                <div class="sidebar-name">
                                    <div class="sidebar-name-arrow"><br /></div>
                                    <h3><?php echo "<strong>" . __('Active widgets', 'everyboard') . "</strong> - ".esc_html( $registered_sidebar['name'] ); ?> <span class="spinner"></span></h3></div>
                                <div class="widget-holder inactive">
                                    <br />
                                    <div id='wp_inactive_widgets' class=''>
                                        <?php wp_list_widget_controls( $sidebar ); // Show the control forms for each of the widgets in this sidebar ?>
                                        <div class="clear"></div>
                                    </div>
                                </div>
                            </div>

                        </div>
                    </div>


                </div>
            </div>
            <?php

            $other_sidebar = $wp_registered_sidebars;
            unset($other_sidebar["everyboard_widgets"]);

            echo '<div style="display: none;">';
            foreach($other_sidebar as $sidebaren) {
                wp_list_widget_controls( $sidebaren["id"] );
            }
            echo '</div>';

            // Reset widget names
            foreach($wp_registered_widgets as &$widget) {
                if(isset($widget["tmpname"])) {

                    $widget["name"] = $widget["tmpname"];
                }
            }
            ?>

            <div class="widget-liquid-right">

                <div id="widgets-right">
                    <div class="widgets-holder-wrap" id="remove_widget" style="background: #f9f9f9; border: 1px solid #ccc; border-radius: 3px; padding: 15px 10px 15px 10px;">
                        <em><?php _e( 'Drag prefabed widgets here to delete them', 'everyboard' ); ?></em>
                    </div>

                    <div id="available-widgets" class="widgets-holder-wrap">
                        <div class="sidebar-name">
                            <!--<div class="sidebar-name-arrow"><br /></div>-->
                            <h3><?php echo __('Available widgets', 'everyboard'); ?></h3></div>
                        <div class="widget-holder">
                            <input type="text" placeholder="<?php echo __('Filter widgets', 'everyboard'); ?>" id="filter_widgets" /><br /><br />
                            <!--<p class="description">Dra widgets hit från en panel till höger för att aktivera dem. Dra dem tillbaka till höger för att inaktivera dem och radera dess inställningar.</p>-->
                            <div id="widget-list">
                                <?php wp_list_widgets(); ?>
                            </div>
                        </div>
                    </div>
                </div>
            </div>

        </div>
        <?php
    }
}

global $widgetprefab;
$widgetprefab = new EveryBoard_WidgetPrefab();